@@ -10,6 +10,7 @@
 #  License for the specific language governing permissions and limitations
 #  under the License.
 
+from neutron_lib import policy as neutron_policy
 from oslo_log import versionutils
 from oslo_policy import policy
 
@@ -18,6 +19,24 @@ from neutron.conf.policies import base
 
 COLLECTION_PATH = '/trunks'
 RESOURCE_PATH = '/trunks/{id}'
+TAGS_PATH = RESOURCE_PATH + '/tags'
+TAG_PATH = RESOURCE_PATH + '/tags/{tag_id}'
+
+ACTION_GET_TAGS = [
+    {'method': 'GET', 'path': TAGS_PATH},
+    {'method': 'GET', 'path': TAG_PATH},
+]
+ACTION_PUT_TAGS = [
+    {'method': 'PUT', 'path': TAGS_PATH},
+    {'method': 'PUT', 'path': TAG_PATH},
+]
+ACTION_POST_TAGS = [
+    {'method': 'POST', 'path': TAGS_PATH},
+]
+ACTION_DELETE_TAGS = [
+    {'method': 'DELETE', 'path': TAGS_PATH},
+    {'method': 'DELETE', 'path': TAG_PATH},
+]
 
 DEPRECATED_REASON = (
     "The trunks API now supports system scope and default roles.")
@@ -37,10 +56,17 @@ rules = [
         ],
         deprecated_rule=policy.DeprecatedRule(
             name='create_trunk',
-            check_str=base.RULE_ANY,
+            check_str=neutron_policy.RULE_ANY,
             deprecated_reason=DEPRECATED_REASON,
             deprecated_since=versionutils.deprecated.WALLABY)
     ),
+    policy.DocumentedRuleDefault(
+        name='create_trunks_tags',
+        check_str=base.ADMIN_OR_PROJECT_MEMBER,
+        scope_types=['project'],
+        description='Create the trunk tags',
+        operations=ACTION_POST_TAGS
+    ),
     policy.DocumentedRuleDefault(
         name='get_trunk',
         check_str=base.ADMIN_OR_PROJECT_READER,
@@ -58,10 +84,17 @@ rules = [
         ],
         deprecated_rule=policy.DeprecatedRule(
             name='get_trunk',
-            check_str=base.RULE_ADMIN_OR_OWNER,
+            check_str=neutron_policy.RULE_ADMIN_OR_OWNER,
             deprecated_reason=DEPRECATED_REASON,
             deprecated_since=versionutils.deprecated.WALLABY)
     ),
+    policy.DocumentedRuleDefault(
+        name='get_trunks_tags',
+        check_str=base.ADMIN_OR_PROJECT_READER,
+        scope_types=['project'],
+        description='Get the trunk tags',
+        operations=ACTION_GET_TAGS
+    ),
     policy.DocumentedRuleDefault(
         name='update_trunk',
         check_str=base.ADMIN_OR_PROJECT_MEMBER,
@@ -75,10 +108,17 @@ rules = [
         ],
         deprecated_rule=policy.DeprecatedRule(
             name='update_trunk',
-            check_str=base.RULE_ADMIN_OR_OWNER,
+            check_str=neutron_policy.RULE_ADMIN_OR_OWNER,
             deprecated_reason=DEPRECATED_REASON,
             deprecated_since=versionutils.deprecated.WALLABY)
     ),
+    policy.DocumentedRuleDefault(
+        name='update_trunks_tags',
+        check_str=base.ADMIN_OR_PROJECT_MEMBER,
+        scope_types=['project'],
+        description='Update the trunk tags',
+        operations=ACTION_PUT_TAGS
+    ),
     policy.DocumentedRuleDefault(
         name='delete_trunk',
         check_str=base.ADMIN_OR_PROJECT_MEMBER,
@@ -92,10 +132,17 @@ rules = [
         ],
         deprecated_rule=policy.DeprecatedRule(
             name='delete_trunk',
-            check_str=base.RULE_ADMIN_OR_OWNER,
+            check_str=neutron_policy.RULE_ADMIN_OR_OWNER,
             deprecated_reason=DEPRECATED_REASON,
             deprecated_since=versionutils.deprecated.WALLABY)
     ),
+    policy.DocumentedRuleDefault(
+        name='delete_trunks_tags',
+        check_str=base.ADMIN_OR_PROJECT_MEMBER,
+        scope_types=['project'],
+        description='Delete a trunk',
+        operations=ACTION_DELETE_TAGS
+    ),
     policy.DocumentedRuleDefault(
         name='get_subports',
         check_str=base.ADMIN_OR_PROJECT_READER,
@@ -109,7 +156,7 @@ rules = [
         ],
         deprecated_rule=policy.DeprecatedRule(
             name='get_subports',
-            check_str=base.RULE_ANY,
+            check_str=neutron_policy.RULE_ANY,
             deprecated_reason=DEPRECATED_REASON,
             deprecated_since=versionutils.deprecated.WALLABY)
     ),
@@ -126,7 +173,7 @@ rules = [
         ],
         deprecated_rule=policy.DeprecatedRule(
             name='add_subports',
-            check_str=base.RULE_ADMIN_OR_OWNER,
+            check_str=neutron_policy.RULE_ADMIN_OR_OWNER,
             deprecated_reason=DEPRECATED_REASON,
             deprecated_since=versionutils.deprecated.WALLABY)
     ),
@@ -143,7 +190,7 @@ rules = [
         ],
         deprecated_rule=policy.DeprecatedRule(
             name='remove_subports',
-            check_str=base.RULE_ADMIN_OR_OWNER,
+            check_str=neutron_policy.RULE_ADMIN_OR_OWNER,
             deprecated_reason=DEPRECATED_REASON,
             deprecated_since=versionutils.deprecated.WALLABY)
     ),
