@@ -51,13 +51,17 @@ from sqlalchemy import not_
 from neutron._i18n import _
 from neutron.api.rpc.agentnotifiers import l3_rpc_agent_api
 from neutron.common import _constants
+from neutron.common import experimental
 from neutron.common import ipv6_utils
 from neutron.common import utils
+from neutron.conf import experimental as c_exp
 from neutron.db import db_base_plugin_common
 from neutron.db import ipam_pluggable_backend
 from neutron.db import models_v2
 from neutron.db import rbac_db_mixin as rbac_mixin
+from neutron.db import rbac_db_models
 from neutron.db import standardattrdescription_db as stattr_db
+from neutron.exceptions import mtu as mtu_exc
 from neutron.extensions import subnetpool_prefix_ops
 from neutron import ipam
 from neutron.ipam import exceptions as ipam_exc
@@ -162,7 +166,17 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
             result_filters=None)
         return super(NeutronDbPluginV2, cls).__new__(cls, *args, **kwargs)
 
+    @staticmethod
+    def _validate_ipv6_pd():
+        try:
+            if cfg.CONF.ipv6_pd_enabled:
+                experimental.validate_experimental_enabled(
+                    c_exp.EXPERIMENTAL_IPV6_PD)
+        except cfg.NoSuchOptError:
+            pass
+
     def __init__(self):
+        self._validate_ipv6_pd()
         self.set_ipam_backend()
         if (cfg.CONF.notify_nova_on_port_status_changes or
                 cfg.CONF.notify_nova_on_port_data_changes):
@@ -210,7 +224,8 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
         policy = (payload.request_body if event == events.BEFORE_CREATE
                   else payload.latest_state)
 
-        if object_type != 'network' or policy['action'] != 'access_as_shared':
+        if (object_type != 'network' or
+                policy['action'] != rbac_db_models.ACCESS_SHARED):
             # we only care about shared network policies
             return
         # The object a policy targets cannot be changed so we can look
@@ -246,26 +261,32 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
         with db_api.CONTEXT_READER.using(elevated):
             ports = model_query.query_with_hooks(
                 elevated, models_v2.Port).filter(
-                models_v2.Port.network_id == network_id)
+                    models_v2.Port.network_id == network_id)
             if tenant_id == '*':
                 # for the wildcard we need to get all of the rbac entries to
                 # see if any allow the remaining ports on the network.
                 # any port with another RBAC entry covering it or one belonging
                 # to the same tenant as the network owner is ok
                 other_rbac_objs = network_obj.NetworkRBAC.get_objects(
-                    elevated, object_id=network_id, action='access_as_shared')
+                    elevated, object_id=network_id,
+                    action=rbac_db_models.ACCESS_SHARED)
                 allowed_tenants = [rbac['target_project'] for rbac
                                    in other_rbac_objs
                                    if rbac.target_project != tenant_id]
                 allowed_tenants.append(net_tenant_id)
                 ports = ports.filter(
                     ~models_v2.Port.tenant_id.in_(allowed_tenants))
+                # Filter any port with project_id=''. These ports are related
+                # to floating IPs, router ports (gateway, SNAT, FIP agent, HA
+                # interface).
+                ports = ports.filter(models_v2.Port.project_id.notin_(['']))
             else:
                 # if there is a wildcard rule, we can return early because it
                 # allows any ports
                 if network_obj.NetworkRBAC.get_object(
                         elevated, object_id=network_id,
-                        action='access_as_shared', target_project='*'):
+                        action=rbac_db_models.ACCESS_SHARED,
+                        target_project='*'):
                     return
                 ports = ports.filter(models_v2.Port.project_id == tenant_id)
             if ports.count():
@@ -311,7 +332,8 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
     def _validate_projects_have_access_to_network(self, network, project_ids):
         ctx_admin = ctx.get_admin_context()
         other_rbac_objs = network_obj.NetworkRBAC.get_objects(
-            ctx_admin, object_id=network.id, action='access_as_shared')
+            ctx_admin, object_id=network.id,
+            action=rbac_db_models.ACCESS_SHARED)
         allowed_projects = {rbac['target_project'] for rbac in other_rbac_objs
                             if rbac.target_project != '*'}
         allowed_projects.add(network.project_id)
@@ -425,7 +447,7 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
             if n['shared']:
                 np_rbac_args = {'project_id': network.project_id,
                                 'object_id': network.id,
-                                'action': 'access_as_shared',
+                                'action': rbac_db_models.ACCESS_SHARED,
                                 'target_project': '*'}
                 np_rbac_obj = network_obj.NetworkRBAC(context, **np_rbac_args)
                 np_rbac_obj.create()
@@ -443,7 +465,7 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
             if 'shared' in n:
                 entry = None
                 for item in network.rbac_entries:
-                    if (item.action == 'access_as_shared' and
+                    if (item.action == rbac_db_models.ACCESS_SHARED and
                             item.target_project == '*'):
                         entry = item
                         break
@@ -453,7 +475,7 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
                 if update_shared and not entry:
                     np_rbac_args = {'project_id': network.project_id,
                                     'object_id': network.id,
-                                    'action': 'access_as_shared',
+                                    'action': rbac_db_models.ACCESS_SHARED,
                                     'target_project': '*'}
                     np_rbac_obj = network_obj.NetworkRBAC(context,
                                                           **np_rbac_args)
@@ -461,7 +483,8 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
                 elif not update_shared and entry:
                     network_obj.NetworkRBAC.delete_objects(
                         context, object_id=network.id,
-                        action='access_as_shared', target_project='*')
+                        action=rbac_db_models.ACCESS_SHARED,
+                        target_project='*')
 
                 # TODO(ihrachys) Below can be removed when we make sqlalchemy
                 # event listeners in neutron_lib/db/api.py to refresh expired
@@ -473,6 +496,10 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
                 # context.
                 getattr(network, 'rbac_entries')
 
+            # validate 'mtu' parameter
+            if 'mtu' in n:
+                self._validate_change_network_mtu(context, id, n['mtu'])
+
             # The filter call removes attributes from the body received from
             # the API that are logically tied to network resources but are
             # stored in other database tables handled by extensions
@@ -480,11 +507,33 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
                 ndb_utils.filter_non_model_columns(n, models_v2.Network))
         return self._make_network_dict(network, context=context)
 
+    def _validate_change_network_mtu(self, context, id, mtu):
+        # can support either ip_version
+        if mtu >= constants.IPV6_MIN_MTU:
+            return
+
+        subnets = self._get_subnets_by_network(context, id)
+        if len(subnets) == 0:
+            return
+
+        # at least one subnet present, if below IPv4 minimum we fail early
+        if mtu < constants.IPV4_MIN_MTU:
+            raise mtu_exc.NetworkMTUSubnetConflict(
+                net_id=id, mtu=constants.IPV4_MIN_MTU)
+
+        # We do not need to check IPv4 subnets as they will have been
+        # caught by above IPV4_MIN_MTU check
+        for subnet in subnets:
+            if (subnet.ip_version == constants.IP_VERSION_6 and
+                    mtu < constants.IPV6_MIN_MTU):
+                raise mtu_exc.NetworkMTUSubnetConflict(
+                    net_id=id, mtu=constants.IPV6_MIN_MTU)
+
     def _ensure_network_not_in_use(self, context, net_id):
         non_auto_ports = context.session.query(
             models_v2.Port.id).filter_by(network_id=net_id).filter(
-            ~models_v2.Port.device_owner.in_(
-                _constants.AUTO_DELETE_PORT_OWNERS))
+                ~models_v2.Port.device_owner.in_(
+                    _constants.AUTO_DELETE_PORT_OWNERS))
         if non_auto_ports.count():
             ports = [port.id for port in non_auto_ports.all()]
             reason = _("There are one or more ports still in use on the "
@@ -500,8 +549,8 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
             self._ensure_network_not_in_use(context, id)
             auto_delete_port_ids = [p.id for p in context.session.query(
                 models_v2.Port.id).filter_by(network_id=id).filter(
-                models_v2.Port.device_owner.in_(
-                    _constants.AUTO_DELETE_PORT_OWNERS))]
+                    models_v2.Port.device_owner.in_(
+                        _constants.AUTO_DELETE_PORT_OWNERS))]
         for port_id in auto_delete_port_ids:
             try:
                 self.delete_port(context.elevated(), port_id)
@@ -595,7 +644,7 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
                     "the ip_version '%(ip_version)s'") % data
             raise exc.InvalidInput(error_message=msg)
 
-    def _validate_subnet(self, context, s, cur_subnet=None):
+    def _validate_subnet(self, context, s, cur_subnet=None, is_pd=False):
         """Validate a subnet spec."""
 
         # This method will validate attributes which may change during
@@ -613,6 +662,7 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
         has_cidr = False
         if validators.is_attr_set(s.get('cidr')):
             self._validate_ip_version(ip_ver, s['cidr'], 'cidr')
+            net = netaddr.IPNetwork(s['cidr'])
             has_cidr = True
 
         # TODO(watanabe.isao): After we found a way to avoid the re-sync
@@ -621,15 +671,21 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
             dhcp_was_enabled = cur_subnet.enable_dhcp
         else:
             dhcp_was_enabled = False
+        # A subnet cidr of '::' is invalid, unless the caller has
+        # indicated they are doing Prefix Delegation,
+        # see https://bugs.launchpad.net/neutron/+bug/2028159
+        if (has_cidr and ip_ver == constants.IP_VERSION_6 and
+                net.network == netaddr.IPAddress('::') and not
+                is_pd):
+            error_message = _("IPv6 subnet '::' is not supported")
+            raise exc.InvalidInput(error_message=error_message)
         if has_cidr and s.get('enable_dhcp') and not dhcp_was_enabled:
-            subnet_prefixlen = netaddr.IPNetwork(s['cidr']).prefixlen
             error_message = _("Subnet has a prefix length that is "
                               "incompatible with DHCP service enabled")
-            if ((ip_ver == 4 and subnet_prefixlen > 30) or
-                    (ip_ver == 6 and subnet_prefixlen > 126)):
+            if ((ip_ver == 4 and net.prefixlen > 30) or
+                    (ip_ver == 6 and net.prefixlen > 126)):
                 raise exc.InvalidInput(error_message=error_message)
 
-            net = netaddr.IPNetwork(s['cidr'])
             if net.is_multicast():
                 error_message = _("Multicast IP subnet is not supported "
                                   "if enable_dhcp is True")
@@ -735,42 +791,29 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
                        "Prefix Delegation.")
             raise exc.BadRequest(resource='subnets', msg=reason)
 
+    def _validate_subnet_network_mtu(self, network, subnet):
+        """Validates that network mtu is correct for subnet association"""
+        mtu = network.mtu
+        if not mtu or mtu >= constants.IPV6_MIN_MTU:
+            return
+
+        # if below IPv4 minimum we fail early
+        if mtu < constants.IPV4_MIN_MTU:
+            raise mtu_exc.NetworkMTUSubnetConflict(net_id=network.id, mtu=mtu)
+
+        # We do not need to check IPv4 subnets as they will have been
+        # caught by above IPV4_MIN_MTU check
+        ip_version = subnet.get('ip_version')
+        if (ip_version == constants.IP_VERSION_6 and
+                mtu < constants.IPV6_MIN_MTU):
+            raise mtu_exc.NetworkMTUSubnetConflict(net_id=network.id, mtu=mtu)
+
     def _update_router_gw_ports(self, context, network, subnet):
         l3plugin = directory.get_plugin(plugin_constants.L3)
-        if l3plugin:
-            gw_ports = self._get_router_gw_ports_by_network(context,
-                                                            network['id'])
-            router_ids = [p.device_id for p in gw_ports]
-            for id in router_ids:
-                try:
-                    self._update_router_gw_port(context, id, network, subnet)
-                except l3_exc.RouterNotFound:
-                    LOG.debug("Router %(id)s was concurrently deleted while "
-                              "updating GW port for subnet %(s)s",
-                              {'id': id, 's': subnet})
-
-    def _update_router_gw_port(self, context, router_id, network, subnet):
-        l3plugin = directory.get_plugin(plugin_constants.L3)
-        ctx_admin = context.elevated()
-        ext_subnets_dict = {s['id']: s for s in network['subnets']}
-        router = l3plugin.get_router(ctx_admin, router_id)
-        external_gateway_info = router['external_gateway_info']
-        # Get all stateful (i.e. non-SLAAC/DHCPv6-stateless) fixed ips
-        fips = [f for f in external_gateway_info['external_fixed_ips']
-                if not ipv6_utils.is_auto_address_subnet(
-                    ext_subnets_dict[f['subnet_id']])]
-        num_fips = len(fips)
-        # Don't add the fixed IP to the port if it already
-        # has a stateful fixed IP of the same IP version
-        if num_fips > 1:
-            return
-        if num_fips == 1 and netaddr.IPAddress(
-                fips[0]['ip_address']).version == subnet['ip_version']:
-            return
-        external_gateway_info['external_fixed_ips'].append(
-                                     {'subnet_id': subnet['id']})
-        info = {'router': {'external_gateway_info': external_gateway_info}}
-        l3plugin.update_router(ctx_admin, router_id, info)
+        # The hasattr check for customized l3 plugins that may have no such
+        # function.
+        if l3plugin and hasattr(l3plugin, "update_router_gw_ports"):
+            l3plugin.update_router_gw_ports(context, network, subnet)
 
     @db_api.retry_if_session_inactive()
     def _create_subnet_postcommit(self, context, result,
@@ -874,9 +917,11 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
             raise exc.BadRequest(resource='subnets', msg=msg)
 
         validate = True
+        is_pd = False
         if subnetpool_id:
             self.ipam.validate_pools_with_subnetpool(s)
             if subnetpool_id == constants.IPV6_PD_POOL_ID:
+                is_pd = True
                 if has_cidr:
                     # We do not currently support requesting a specific
                     # cidr with IPv6 prefix delegation. Set the subnetpool_id
@@ -894,11 +939,12 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
                 raise exc.BadRequest(resource='subnets', msg=msg)
 
         if validate:
-            self._validate_subnet(context, s)
+            self._validate_subnet(context, s, is_pd=is_pd)
 
         with db_api.CONTEXT_WRITER.using(context):
             network = self._get_network(context,
                                         subnet['subnet']['network_id'])
+            self._validate_subnet_network_mtu(network, s)
             subnet, ipam_subnet = self.ipam.allocate_subnet(context,
                                                             network,
                                                             subnet['subnet'],
@@ -950,7 +996,8 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
         # Fill 'network_id' field with the current value since this is expected
         # by _validate_segment() in ipam_pluggable_backend.
         s['network_id'] = subnet_obj.network_id
-        self._validate_subnet(context, s, cur_subnet=subnet_obj)
+        is_pd = s['subnetpool_id'] == constants.IPV6_PD_POOL_ID
+        self._validate_subnet(context, s, cur_subnet=subnet_obj, is_pd=is_pd)
         db_pools = [netaddr.IPRange(p.start, p.end)
                     for p in subnet_obj.allocation_pools]
 
@@ -1076,7 +1123,7 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
         # Do not allow a subnet to be deleted if a router is attached to it
         sid = subnet['id']
         self._subnet_check_ip_allocations_internal_router_ports(
-                context, sid)
+            context, sid)
         is_auto_addr_subnet = ipv6_utils.is_auto_address_subnet(subnet)
         if not is_auto_addr_subnet:
             # we only automatically remove IP addresses from user ports if
@@ -1377,9 +1424,9 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
             raise exc.SubnetPoolNotFound(subnetpool_id=id)
 
         subnets_to_onboard = subnet_obj.Subnet.get_objects(
-                                             context,
-                                             network_id=network_id,
-                                             ip_version=subnetpool.ip_version)
+            context,
+            network_id=network_id,
+            ip_version=subnetpool.ip_version)
 
         self._onboard_network_subnets(context, subnets_to_onboard, subnetpool)
 
@@ -1401,8 +1448,8 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
                                  subnetpool):
         allocated_prefix_set = netaddr.IPSet(
             [x.cidr for x in subnet_obj.Subnet.get_objects(
-                                                context,
-                                                subnetpool_id=subnetpool.id)])
+                context,
+                subnetpool_id=subnetpool.id)])
         prefixes_to_add = []
 
         for subnet in subnets_to_onboard:
@@ -1428,8 +1475,8 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
 
     def _check_mac_addr_update(self, context, port, new_mac, device_owner):
         if (device_owner and
-            device_owner.startswith(
-                constants.DEVICE_OWNER_NETWORK_PREFIX)):
+                device_owner.startswith(
+                    constants.DEVICE_OWNER_NETWORK_PREFIX)):
             raise exc.UnsupportedPortDeviceOwner(
                 op=_("mac address update"), port_id=id,
                 device_owner=device_owner)
@@ -1583,8 +1630,8 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
         port = self._get_port(context, id, lazy_fields=lazy_fields)
         return self._make_port_dict(port, fields)
 
-    def _get_ports_query(self, context, filters=None, lazy_fields=None,
-                         *args, **kwargs):
+    def _get_ports_query(self, context, *args, filters=None, lazy_fields=None,
+                         **kwargs):
         Port = models_v2.Port
         IPAllocation = models_v2.IPAllocation
 
@@ -1640,7 +1687,7 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
     @db_api.retry_if_session_inactive()
     @db_api.CONTEXT_READER
     def get_ports_count(self, context, filters=None):
-        return self._get_ports_query(context, filters).count()
+        return self._get_ports_query(context, filters=filters).count()
 
     def _enforce_device_owner_not_router_intf_or_device_id(self, context,
                                                            device_owner,
