@@ -14,6 +14,7 @@
 import abc
 import collections
 import copy
+import functools
 import itertools
 
 from neutron_lib.api.definitions import port
@@ -25,7 +26,6 @@ from neutron_lib import exceptions
 from neutron_lib.plugins import directory
 from neutron_lib import rpc as n_rpc
 from neutron_lib.services import base as service_base
-import webob.exc
 
 from neutron._i18n import _
 from neutron.api import extensions
@@ -89,6 +89,14 @@ ResourceInfo = collections.namedtuple(
 EMPTY_RESOURCE_INFO = ResourceInfo(None, None, None, None, None)
 
 
+def _policy_init(f):
+    @functools.wraps(f)
+    def func(self, *args, **kwargs):
+        policy.init()
+        return f(self, *args, **kwargs)
+    return func
+
+
 class TagResourceNotFound(exceptions.NotFound):
     message = _("Resource %(resource)s %(resource_id)s could not be found.")
 
@@ -180,6 +188,7 @@ class TaggingController(object):
         # This should never be returned.
         return EMPTY_RESOURCE_INFO
 
+    @_policy_init
     def index(self, request, **kwargs):
         # GET /v2.0/{parent_resource}/{parent_resource_id}/tags
         ctx = request.context
@@ -189,6 +198,7 @@ class TaggingController(object):
                        target)
         return self.plugin.get_tags(ctx, rinfo.parent_type, rinfo.parent_id)
 
+    @_policy_init
     def show(self, request, id, **kwargs):
         # GET /v2.0/{parent_resource}/{parent_resource_id}/tags/{tag}
         # id == tag
@@ -200,11 +210,25 @@ class TaggingController(object):
                        target)
         return self.plugin.get_tag(ctx, rinfo.parent_type, rinfo.parent_id, id)
 
-    def create(self, request, **kwargs):
-        # not supported
+    @_policy_init
+    def create(self, request, body, **kwargs):
         # POST /v2.0/{parent_resource}/{parent_resource_id}/tags
-        raise webob.exc.HTTPNotFound("not supported")
+        # body: {"tags": ["aaa", "bbb"]}
+        validate_tags(body)
+        ctx = request.context
+        rinfo = self._get_resource_info(ctx, kwargs)
+        target = self._get_target(rinfo)
+        policy.enforce(ctx, 'create_{}_{}'.format(rinfo.parent_type, TAGS),
+                       target)
+        notify_tag_action(ctx, 'create.start', rinfo.parent_type,
+                          rinfo.parent_id, body['tags'])
+        result = self.plugin.create_tags(ctx, rinfo.parent_type,
+                                         rinfo.parent_id, body)
+        notify_tag_action(ctx, 'create.end', rinfo.parent_type,
+                          rinfo.parent_id, body['tags'])
+        return result
 
+    @_policy_init
     def update(self, request, id, **kwargs):
         # PUT /v2.0/{parent_resource}/{parent_resource_id}/tags/{tag}
         # id == tag
@@ -222,6 +246,7 @@ class TaggingController(object):
                           rinfo.parent_id, [id])
         return result
 
+    @_policy_init
     def update_all(self, request, body, **kwargs):
         # PUT /v2.0/{parent_resource}/{parent_resource_id}/tags
         # body: {"tags": ["aaa", "bbb"]}
@@ -239,6 +264,7 @@ class TaggingController(object):
                           rinfo.parent_id, body['tags'])
         return result
 
+    @_policy_init
     def delete(self, request, id, **kwargs):
         # DELETE /v2.0/{parent_resource}/{parent_resource_id}/tags/{tag}
         # id == tag
@@ -256,6 +282,7 @@ class TaggingController(object):
                           rinfo.parent_id, [id])
         return result
 
+    @_policy_init
     def delete_all(self, request, **kwargs):
         # DELETE /v2.0/{parent_resource}/{parent_resource_id}/tags
         ctx = request.context
