@@ -21,6 +21,7 @@ from neutron_lib.api.definitions import trunk_details
 from neutron_lib.callbacks import events
 from neutron_lib.callbacks import registry
 from neutron_lib.callbacks import resources
+from neutron_lib import constants as const
 from neutron_lib import context
 from neutron_lib.db import api as db_api
 from neutron_lib.db import resource_extend
@@ -36,6 +37,8 @@ from neutron.objects import base as objects_base
 from neutron.objects import trunk as trunk_objects
 from neutron.services.trunk import drivers
 from neutron.services.trunk import exceptions as trunk_exc
+from neutron.services.trunk.rpc import backend
+from neutron.services.trunk.rpc import server
 from neutron.services.trunk import rules
 from neutron.services.trunk.seg_types import validators
 
@@ -54,7 +57,8 @@ class TrunkPlugin(service_base.ServicePluginBase):
     __filter_validation_support = True
 
     def __init__(self):
-        self._rpc_backend = None
+        self._rpc_server = None
+        self._rpc_notifier = None
         self._drivers = []
         self._segmentation_types = {}
         self._interfaces = set()
@@ -63,6 +67,10 @@ class TrunkPlugin(service_base.ServicePluginBase):
         registry.subscribe(rules.enforce_port_deletion_rules,
                            resources.PORT, events.BEFORE_DELETE)
         registry.publish(resources.TRUNK_PLUGIN, events.AFTER_INIT, self)
+        if any(drv.rpc_required for drv in self._drivers):
+            # create notifier backend
+            self._rpc_notifier = backend.ServerSideRpcBackend()
+
         for driver in self._drivers:
             LOG.debug('Trunk plugin loaded with driver %s', driver.name)
         self.check_compatibility()
@@ -90,6 +98,13 @@ class TrunkPlugin(service_base.ServicePluginBase):
 
         return port_res
 
+    def start_rpc_listeners(self):
+        if not any(drv.rpc_required for drv in self._drivers):
+            return []
+
+        self._rpc_server = server.TrunkSkeleton()
+        return self._rpc_server.rpc_servers
+
     @staticmethod
     @resource_extend.extends([port_def.COLLECTION_NAME_BULK])
     def _extend_port_trunk_details_bulk(ports_res, noop):
@@ -154,12 +169,6 @@ class TrunkPlugin(service_base.ServicePluginBase):
             raise trunk_exc.SegmentationTypeValidatorNotFound(
                 seg_type=seg_type)
 
-    def set_rpc_backend(self, backend):
-        self._rpc_backend = backend
-
-    def is_rpc_enabled(self):
-        return self._rpc_backend is not None
-
     def register_driver(self, driver):
         """Register driver with trunk plugin."""
         if driver.agent_type:
@@ -463,12 +472,19 @@ class TrunkPlugin(service_base.ServicePluginBase):
         original_port = payload.states[0]
         orig_vif_type = original_port.get(portbindings.VIF_TYPE)
         new_vif_type = updated_port.get(portbindings.VIF_TYPE)
+        orig_status = original_port.get('status')
+        new_status = updated_port.get('status')
         vif_type_changed = orig_vif_type != new_vif_type
+        trunk_id = trunk_details['trunk_id']
         if vif_type_changed and new_vif_type == portbindings.VIF_TYPE_UNBOUND:
-            trunk_id = trunk_details['trunk_id']
             # NOTE(status_police) Trunk status goes to DOWN when the parent
             # port is unbound. This means there are no more physical resources
             # associated with the logical resource.
             self.update_trunk(
                 context, trunk_id,
                 {'trunk': {'status': constants.TRUNK_DOWN_STATUS}})
+        elif new_status == const.PORT_STATUS_ACTIVE and \
+                new_status != orig_status:
+            self.update_trunk(
+                context, trunk_id,
+                {'trunk': {'status': constants.TRUNK_ACTIVE_STATUS}})
