@@ -32,6 +32,7 @@ from ovsdbapp.backend.ovs_idl import idlutils
 
 from neutron.agent.linux import utils
 from neutron.api import extensions as exts
+from neutron.api import wsgi
 from neutron.common import utils as n_utils
 from neutron.conf.agent import common as config
 from neutron.conf.agent import ovs_conf
@@ -53,7 +54,6 @@ from neutron.tests.common import helpers
 from neutron.tests.functional.resources import process
 from neutron.tests.unit.extensions import test_securitygroup
 from neutron.tests.unit.plugins.ml2 import test_plugin
-import neutron.wsgi
 
 LOG = log.getLogger(__name__)
 
@@ -140,7 +140,7 @@ class TestOVNFunctionalBase(test_plugin.Ml2PluginV2TestCase,
     OVN_SCHEMA_FILES = ['ovn-nb.ovsschema', 'ovn-sb.ovsschema']
 
     _mechanism_drivers = ['logger', 'ovn']
-    _extension_drivers = ['port_security']
+    _extension_drivers = ['port_security', 'external-gateway-multihoming']
     _counter = 0
     l3_plugin = 'neutron.services.ovn_l3.plugin.OVNL3RouterPlugin'
 
@@ -329,7 +329,7 @@ class TestOVNFunctionalBase(test_plugin.Ml2PluginV2TestCase,
             trigger_cls.trigger.__self__.__class__ = worker.MaintenanceWorker
             cfg.CONF.set_override('neutron_sync_mode', 'off', 'ovn')
         else:
-            trigger_cls.trigger.__self__.__class__ = neutron.wsgi.WorkerService
+            trigger_cls.trigger.__self__.__class__ = wsgi.WorkerService
 
         self.addCleanup(self.stop)
         # NOTE(ralonsoh): do not access to the DB at exit when the SQL
@@ -359,7 +359,13 @@ class TestOVNFunctionalBase(test_plugin.Ml2PluginV2TestCase,
                     'suffix': file_suffix,
                     'timestamp': timestamp,
                 }
-                self._copy_log_file(src_filename, dst_filename)
+                try:
+                    self._copy_log_file(src_filename, dst_filename)
+                except FileNotFoundError:
+                    # Some testcases add the method ``_collect_processes_logs``
+                    # twice in the cleanup methods. The second time this method
+                    # is called, the logs and DBs have been already deleted.
+                    pass
 
         # Copy northd logs
         northd_log = "ovn_northd"
@@ -438,24 +444,22 @@ class TestOVNFunctionalBase(test_plugin.Ml2PluginV2TestCase,
             name, ['geneve'], '172.24.4.%d' % self._counter,
             external_ids=external_ids, hostname=host,
             other_config=other_config).execute(check_error=True)
-        if self.sb_api.is_table_present('Chassis_Private'):
-            nb_cfg_timestamp = timeutils.utcnow_ts() * 1000
-            self.sb_api.db_create(
-                'Chassis_Private', name=name, external_ids=external_ids,
-                chassis=chassis.uuid, nb_cfg_timestamp=nb_cfg_timestamp
-            ).execute(check_error=True)
+        nb_cfg_timestamp = timeutils.utcnow_ts() * 1000
+        self.sb_api.db_create(
+            'Chassis_Private', name=name, external_ids=external_ids,
+            chassis=chassis.uuid, nb_cfg_timestamp=nb_cfg_timestamp
+        ).execute(check_error=True)
         return name
 
     def del_fake_chassis(self, chassis, if_exists=True):
         self.sb_api.chassis_del(
             chassis, if_exists=if_exists).execute(check_error=True)
         try:
-            if self.sb_api.is_table_present('Chassis_Private'):
-                self.sb_api.db_destroy(
-                    'Chassis_Private', chassis).execute(check_error=True)
+            self.sb_api.db_destroy(
+                'Chassis_Private', chassis).execute(check_error=True)
         except idlutils.RowNotFound:
-            # NOTE(ykarel ): ovsdbapp >= 2.4.2 handles Chassis_Private
+            # NOTE(ykarel ): ovsdbapp >= 2.7.0 handles Chassis_Private
             # record delete with chassis
             # try/except can be dropped when neutron requirements.txt
-            # include ovsdbapp>=2.4.2
+            # include ovsdbapp>=2.7.0
             pass
