@@ -12,6 +12,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from collections import defaultdict
 from collections import namedtuple
 
 import netaddr
@@ -23,13 +24,11 @@ from neutron_lib.api.definitions import port_security as ps
 from neutron_lib import constants
 from neutron_lib import context
 from neutron_lib.db import api as db_api
-from neutron_lib.services.logapi import constants as log_const
 from neutron_lib.services.qos import constants as qos_const
 from oslo_config import cfg
 from oslo_utils import uuidutils
 from ovsdbapp.backend.ovs_idl import idlutils
 from ovsdbapp import constants as ovsdbapp_const
-from sqlalchemy.dialects.mysql import dialect as mysql_dialect
 
 from neutron.common.ovn import acl as acl_utils
 from neutron.common.ovn import constants as ovn_const
@@ -49,14 +48,13 @@ from neutron.tests.unit.extensions import test_securitygroup
 from neutron.tests.unit import testlib_api
 
 
-class TestOvnNbSync(testlib_api.MySQLTestCaseMixin,
-                    base.TestOVNFunctionalBase):
+class TestOvnNbSync(base.TestOVNFunctionalBase,
+                    testlib_api.MySQLTestCaseMixin):
 
     _extension_drivers = ['port_security', 'dns', 'qos', 'revision_plugin']
 
     def setUp(self, *args):
         super(TestOvnNbSync, self).setUp(maintenance_worker=True)
-        self.assertEqual(mysql_dialect.name, self.db.engine.dialect.name)
         ovn_config.cfg.CONF.set_override('dns_domain', 'ovn.test')
         cfg.CONF.set_override('quota_security_group_rule', -1, group='QUOTAS')
         ext_mgr = test_extraroute.ExtraRouteTestExtensionManager()
@@ -107,7 +105,7 @@ class TestOvnNbSync(testlib_api.MySQLTestCaseMixin,
             nb_idl=self.nb_api)
 
     def get_additional_service_plugins(self):
-        return {'qos': 'qos', 'segments': 'segments', 'log': 'log'}
+        return {'qos': 'qos', 'segments': 'segments'}
 
     def _api_for_resource(self, resource):
         if resource in ['security-groups']:
@@ -801,14 +799,19 @@ class TestOvnNbSync(testlib_api.MySQLTestCaseMixin,
                 txn.add(self.nb_api.delete_lrouter_port(lrport,
                                                         lrouter_name, True))
 
+            columns = {'external_ids': {ovn_const.OVN_LRSR_EXT_ID_KEY: 'true'}}
             for lrouter_name, ip_prefix, nexthop in self.create_lrouter_routes:
                 txn.add(self.nb_api.add_static_route(lrouter_name,
                                                      ip_prefix=ip_prefix,
-                                                     nexthop=nexthop))
-
+                                                     nexthop=nexthop,
+                                                     **columns))
+            routers = defaultdict(list)
             for lrouter_name, ip_prefix, nexthop in self.delete_lrouter_routes:
-                txn.add(self.nb_api.delete_static_route(lrouter_name,
-                                                        ip_prefix, nexthop,
+                routers[lrouter_name].append((ip_prefix, nexthop))
+
+            for lrouter_name, routes_to_delete in routers.items():
+                txn.add(self.nb_api.delete_static_routes(lrouter_name,
+                                                        routes_to_delete,
                                                         True))
 
             for lrouter_name, nat_dict in self.create_lrouter_nats:
@@ -1076,7 +1079,7 @@ class TestOvnNbSync(testlib_api.MySQLTestCaseMixin,
 
     @staticmethod
     def _build_acl_for_pgs(priority, direction, log, name, action,
-                           severity, match, meter, port_group, **kwargs):
+                           severity, match, port_group, **kwargs):
         return {
             'priority': priority,
             'direction': direction,
@@ -1085,7 +1088,6 @@ class TestOvnNbSync(testlib_api.MySQLTestCaseMixin,
             'action': action,
             'severity': severity,
             'match': match,
-            'meter': meter,
             'external_ids': kwargs}
 
     def _validate_dhcp_opts(self, should_match=True):
@@ -1158,9 +1160,6 @@ class TestOvnNbSync(testlib_api.MySQLTestCaseMixin,
                 ovn_const.OVN_DROP_PORT_GROUP_NAME):
             db_acls.append(TestOvnNbSync._build_acl_for_pgs(**acl))
 
-        self.ovn_log_driver.add_logging_options_to_acls(db_acls,
-                                                        self.ctx)
-
         # Get the list of ACLs stored in the OVN plugin IDL.
         plugin_acls = []
         for row in _plugin_nb_ovn._tables['Logical_Switch'].rows.values():
@@ -1181,24 +1180,7 @@ class TestOvnNbSync(testlib_api.MySQLTestCaseMixin,
             for acl in getattr(row, 'acls', []):
                 monitor_acls.append(self._build_acl_to_compare(acl))
 
-        self.ovn_log_driver.add_logging_options_to_acls(monitor_acls,
-                                                        self.ctx)
-        self.ovn_log_driver.add_logging_options_to_acls(plugin_acls,
-                                                        self.ctx)
-
-        # Values taken out from list for comparison, since ACLs from OVN DB
-        # have certain values on a list of just one object
         if should_match:
-            for acl in plugin_acls:
-                if isinstance(acl['severity'], list) and acl['severity']:
-                    acl['severity'] = acl['severity'][0]
-                    acl['name'] = acl['name'][0]
-                    acl['meter'] = acl['meter'][0]
-            for acl in monitor_acls:
-                if isinstance(acl['severity'], list) and acl['severity']:
-                    acl['severity'] = acl['severity'][0]
-                    acl['name'] = acl['name'][0]
-                    acl['meter'] = acl['meter'][0]
             self.assertCountEqual(db_acls, plugin_acls)
             self.assertCountEqual(db_acls, monitor_acls)
         else:
@@ -1792,25 +1774,13 @@ class TestOvnNbSync(testlib_api.MySQLTestCaseMixin,
         self.assertIn('security_group_rule', sgr)
         return sgr['security_group_rule']['id']
 
-    def _test_sync_acls_helper(self, test_log=False,
-                               log_event=log_const.ALL_EVENT):
+    def test_sync_acls(self):
         data = {'security_group': {'name': 'sg1'}}
         sg_req = self.new_create_request('security-groups', data)
         res = sg_req.get_response(self.api)
         sg = self.deserialize(self.fmt, res)['security_group']
 
         sgr_ids = []
-
-        # If we are going to test ACLs with log enabled, set up a log object
-        if test_log:
-            log_data = {'log': {'project_id': self.ctx.project_id,
-            'resource_type': 'security_group',
-            'description': 'test net log',
-            'name': 'logme',
-            'enabled': True,
-            'event': log_event}}
-            log_obj = self.log_plugin.create_log(self.ctx, log_data)
-
         for tcp_port in range(8050, 8055):
             sgr_ids.append(self._create_security_group_rule(
                 sg['id'], 'ingress', tcp_port))
@@ -1841,21 +1811,6 @@ class TestOvnNbSync(testlib_api.MySQLTestCaseMixin,
         nb_synchronizer.sync_acls(ctx)
         self._validate_acls()
 
-        # Remove log object to avoid overlapping
-        if test_log:
-            log_obj = self.log_plugin.delete_log(self.ctx, log_obj['id'])
-
-    def test_sync_acls(self):
-        self._test_sync_acls_helper()
-
-    def test_sync_acls_with_logging(self):
-        self._test_sync_acls_helper(test_log=True,
-                                    log_event=log_const.ACCEPT_EVENT)
-        self._test_sync_acls_helper(test_log=True,
-                                    log_event=log_const.ALL_EVENT)
-        self._test_sync_acls_helper(test_log=True,
-                                    log_event=log_const.DROP_EVENT)
-
 
 class TestOvnSbSync(base.TestOVNFunctionalBase):
 
@@ -1865,7 +1820,6 @@ class TestOvnSbSync(base.TestOVNFunctionalBase):
             self.plugin, self.mech_driver.sb_ovn, self.mech_driver)
         self.addCleanup(self.sb_synchronizer.stop)
         self.ctx = context.get_admin_context()
-        self.host1 = uuidutils.generate_uuid()
 
     def _sync_resources(self):
         self.sb_synchronizer.sync_hostname_and_physical_networks(self.ctx)
@@ -1899,23 +1853,23 @@ class TestOvnSbSync(base.TestOVNFunctionalBase):
     def test_ovn_sb_sync_add_new_host(self):
         with self.network() as network:
             network_id = network['network']['id']
-        self.create_segment(network_id, self.physnet, 50)
-        self.add_fake_chassis(self.host1, [self.physnet])
+        self.create_segment(network_id, 'physnet1', 50)
+        self.add_fake_chassis('host1', ['physnet1'])
         segment_hosts = segments_db.get_hosts_mapped_with_segments(self.ctx)
         self.assertFalse(segment_hosts)
         self._sync_resources()
         segment_hosts = segments_db.get_hosts_mapped_with_segments(self.ctx)
-        self.assertEqual({self.host1}, segment_hosts)
+        self.assertEqual({'host1'}, segment_hosts)
 
     def test_ovn_sb_sync_update_existing_host(self):
         with self.network() as network:
             network_id = network['network']['id']
-        segment = self.create_segment(network_id, self.physnet, 50)
+        segment = self.create_segment(network_id, 'physnet1', 50)
         segments_db.update_segment_host_mapping(
-            self.ctx, self.host1, {segment['id']})
+            self.ctx, 'host1', {segment['id']})
         segment_hosts = segments_db.get_hosts_mapped_with_segments(self.ctx)
-        self.assertEqual({self.host1}, segment_hosts)
-        self.add_fake_chassis(self.host1, [self.physnet2])
+        self.assertEqual({'host1'}, segment_hosts)
+        self.add_fake_chassis('host1', ['physnet2'])
         self._sync_resources()
         segment_hosts = segments_db.get_hosts_mapped_with_segments(self.ctx)
         self.assertFalse(segment_hosts)
@@ -1923,14 +1877,13 @@ class TestOvnSbSync(base.TestOVNFunctionalBase):
     def test_ovn_sb_sync_delete_stale_host(self):
         with self.network() as network:
             network_id = network['network']['id']
-        segment = self.create_segment(network_id, self.physnet, 50)
+        segment = self.create_segment(network_id, 'physnet1', 50)
         segments_db.update_segment_host_mapping(
-            self.ctx, self.host1, {segment['id']})
-        _ = self.create_agent(self.host1,
-                              bridge_mappings={self.physnet: 'eth0'})
+            self.ctx, 'host1', {segment['id']})
+        _ = self.create_agent('host1', bridge_mappings={'physnet1': 'eth0'})
         segment_hosts = segments_db.get_hosts_mapped_with_segments(self.ctx)
-        self.assertEqual({self.host1}, segment_hosts)
-        # Since there is no chassis in the sb DB, self.host1 is the stale host
+        self.assertEqual({'host1'}, segment_hosts)
+        # Since there is no chassis in the sb DB, host1 is the stale host
         # recorded in neutron DB. It should be deleted after sync.
         self._sync_resources()
         segment_hosts = segments_db.get_hosts_mapped_with_segments(self.ctx)
@@ -1939,61 +1892,57 @@ class TestOvnSbSync(base.TestOVNFunctionalBase):
     def test_ovn_sb_sync_host_with_no_agent_not_deleted(self):
         with self.network() as network:
             network_id = network['network']['id']
-        segment = self.create_segment(network_id, self.physnet, 50)
+        segment = self.create_segment(network_id, 'physnet1', 50)
         segments_db.update_segment_host_mapping(
-            self.ctx, self.host1, {segment['id']})
-        _ = self.create_agent(self.host1,
-                              bridge_mappings={self.physnet: 'eth0'},
+            self.ctx, 'host1', {segment['id']})
+        _ = self.create_agent('host1', bridge_mappings={'physnet1': 'eth0'},
                               agent_type="Not OVN Agent")
         segment_hosts = segments_db.get_hosts_mapped_with_segments(self.ctx)
-        self.assertEqual({self.host1}, segment_hosts)
-        # There is no chassis in the sb DB, self.host1 does not have an agent
+        self.assertEqual({'host1'}, segment_hosts)
+        # There is no chassis in the sb DB, host1 does not have an agent
         # so it is not deleted.
         self._sync_resources()
         segment_hosts = segments_db.get_hosts_mapped_with_segments(self.ctx)
-        self.assertEqual({self.host1}, segment_hosts)
+        self.assertEqual({'host1'}, segment_hosts)
 
     def test_ovn_sb_sync_host_with_other_agent_type_not_deleted(self):
         with self.network() as network:
             network_id = network['network']['id']
-        segment = self.create_segment(network_id, self.physnet, 50)
+        segment = self.create_segment(network_id, 'physnet1', 50)
         segments_db.update_segment_host_mapping(
-            self.ctx, self.host1, {segment['id']})
+            self.ctx, 'host1', {segment['id']})
         segment_hosts = segments_db.get_hosts_mapped_with_segments(self.ctx)
-        self.assertEqual({self.host1}, segment_hosts)
-        # There is no chassis in the sb DB, self.host1 does not have an agent
+        self.assertEqual({'host1'}, segment_hosts)
+        # There is no chassis in the sb DB, host1 does not have an agent
         # so it is not deleted.
         self._sync_resources()
         segment_hosts = segments_db.get_hosts_mapped_with_segments(self.ctx)
-        self.assertEqual({self.host1}, segment_hosts)
+        self.assertEqual({'host1'}, segment_hosts)
 
     def test_ovn_sb_sync(self):
-        host2 = uuidutils.generate_uuid()
-        host3 = uuidutils.generate_uuid()
-        host4 = uuidutils.generate_uuid()
         with self.network() as network:
             network_id = network['network']['id']
-        seg1 = self.create_segment(network_id, self.physnet, 50)
-        self.create_segment(network_id, self.physnet2, 51)
+        seg1 = self.create_segment(network_id, 'physnet1', 50)
+        self.create_segment(network_id, 'physnet2', 51)
         segments_db.update_segment_host_mapping(
-            self.ctx, self.host1, {seg1['id']})
+            self.ctx, 'host1', {seg1['id']})
         segments_db.update_segment_host_mapping(
-            self.ctx, host2, {seg1['id']})
+            self.ctx, 'host2', {seg1['id']})
         segments_db.update_segment_host_mapping(
-            self.ctx, host3, {seg1['id']})
+            self.ctx, 'host3', {seg1['id']})
         segment_hosts = segments_db.get_hosts_mapped_with_segments(self.ctx)
-        _ = self.create_agent(self.host1)
-        _ = self.create_agent(host2, bridge_mappings={self.physnet2: 'eth0'})
-        _ = self.create_agent(host3, bridge_mappings={self.physnet3: 'eth0'})
-        self.assertEqual({self.host1, host2, host3}, segment_hosts)
-        self.add_fake_chassis(host2, [self.physnet2])
-        self.add_fake_chassis(host3, [self.physnet3])
-        self.add_fake_chassis(host4, [self.physnet])
+        _ = self.create_agent('host1')
+        _ = self.create_agent('host2', bridge_mappings={'physnet2': 'eth0'})
+        _ = self.create_agent('host3', bridge_mappings={'physnet3': 'eth0'})
+        self.assertEqual({'host1', 'host2', 'host3'}, segment_hosts)
+        self.add_fake_chassis('host2', ['physnet2'])
+        self.add_fake_chassis('host3', ['physnet3'])
+        self.add_fake_chassis('host4', ['physnet1'])
         self._sync_resources()
         segment_hosts = segments_db.get_hosts_mapped_with_segments(self.ctx)
         # host1 should be cleared since it is not in the chassis DB. host3
         # should be cleared since there is no segment for mapping.
-        self.assertEqual({host2, host4}, segment_hosts)
+        self.assertEqual({'host2', 'host4'}, segment_hosts)
 
 
 class TestOvnNbSyncOverTcp(TestOvnNbSync):
