@@ -26,9 +26,11 @@ from neutron.agent.linux.ip_lib import IpAddrCommand as ip_addr
 from neutron.agent.linux.ip_lib import IpLinkCommand as ip_link
 from neutron.agent.linux.ip_lib import IpNetnsCommand as ip_netns
 from neutron.agent.linux.ip_lib import IPWrapper as ip_wrap
+from neutron.agent.linux import utils as linux_utils
 from neutron.agent.ovn.metadata import agent
 from neutron.agent.ovn.metadata import driver
 from neutron.common.ovn import constants as ovn_const
+from neutron.common import utils
 from neutron.conf.agent.metadata import config as meta_conf
 from neutron.conf.agent.ovn.metadata import config as ovn_meta_conf
 from neutron.conf.plugins.ml2.drivers.ovn import ovn_conf
@@ -79,8 +81,8 @@ class TestMetadataAgent(base.BaseTestCase):
         self.agent.sb_idl = mock.Mock()
         self.agent.ovs_idl = mock.Mock()
         self.agent.ovs_idl.transaction = mock.MagicMock()
-        self.agent.chassis = 'chassis'
-        self.agent.ovn_bridge = 'br-int'
+        self.agent._chassis = 'chassis'
+        self.agent._ovn_bridge = 'br-int'
 
         self.ports = []
         for i in range(0, 3):
@@ -97,7 +99,8 @@ class TestMetadataAgent(base.BaseTestCase):
                     ip_lib, 'list_network_namespaces') as lnn,\
                 mock.patch.object(
                     self.agent, 'teardown_datapath') as tdp:
-            lnn.return_value = ['ovnmeta-1', 'ovnmeta-2']
+            lnn.return_value = [ovn_const.OVN_METADATA_PREFIX + '1',
+                                ovn_const.OVN_METADATA_PREFIX + '2']
 
             self.agent.sync()
 
@@ -114,7 +117,9 @@ class TestMetadataAgent(base.BaseTestCase):
                     ip_lib, 'list_network_namespaces') as lnn,\
                 mock.patch.object(
                     self.agent, 'teardown_datapath') as tdp:
-            lnn.return_value = ['ovnmeta-1', 'ovnmeta-2', 'ovnmeta-3',
+            lnn.return_value = [ovn_const.OVN_METADATA_PREFIX + '1',
+                                ovn_const.OVN_METADATA_PREFIX + '2',
+                                ovn_const.OVN_METADATA_PREFIX + '3',
                                 'ns1', 'ns2']
 
             self.agent.sync()
@@ -203,7 +208,7 @@ class TestMetadataAgent(base.BaseTestCase):
             self.agent.teardown_datapath('1')
 
             destroy_mdp.assert_called_once_with(
-                mock.ANY, '1', mock.ANY, 'ovnmeta-1')
+                mock.ANY, '1', mock.ANY, ovn_const.OVN_METADATA_PREFIX + '1')
             self.agent.ovs_idl.del_port.assert_called_once_with('veth_0')
             del_veth.assert_called_once_with('veth_0')
             garbage_collect.assert_called_once_with()
@@ -212,14 +217,18 @@ class TestMetadataAgent(base.BaseTestCase):
         current_namespace_cidrs = set()
         datapath_port_ips = ['10.0.0.2', '10.0.0.3', '10.0.1.5']
         metadaport_subnet_cidrs = ['10.0.0.0/30', '10.0.1.0/28', '11.0.1.2/24']
+        lla = 'fe80::f816:3eff:fe63:8dc5/64'
 
         expected_cidrs_to_add = set(['10.0.0.0/30', '10.0.1.0/28',
-                                     n_const.METADATA_CIDR])
+                                     n_const.METADATA_CIDR,
+                                     n_const.METADATA_V6_CIDR,
+                                     lla])
         expected_cidrs_to_delete = set()
 
         actual_result = self.agent._process_cidrs(current_namespace_cidrs,
                                                   datapath_port_ips,
-                                                  metadaport_subnet_cidrs)
+                                                  metadaport_subnet_cidrs,
+                                                  lla)
         actual_cidrs_to_add, actual_cidrs_to_delete = actual_result
 
         self.assertSetEqual(actual_cidrs_to_add, expected_cidrs_to_add)
@@ -230,29 +239,36 @@ class TestMetadataAgent(base.BaseTestCase):
         current_namespace_cidrs = set([n_const.METADATA_CIDR])
         datapath_port_ips = ['10.0.0.2', '10.0.0.3', '10.0.1.5']
         metadaport_subnet_cidrs = ['10.0.0.0/30', '10.0.1.0/28', '11.0.1.2/24']
+        lla = 'fe80::f816:3eff:fe63:8dc5/64'
 
-        expected_cidrs_to_add = set(['10.0.0.0/30', '10.0.1.0/28'])
+        expected_cidrs_to_add = set(['10.0.0.0/30', '10.0.1.0/28',
+                                     n_const.METADATA_V6_CIDR, lla])
         expected_cidrs_to_delete = set()
 
         actual_result = self.agent._process_cidrs(current_namespace_cidrs,
                                                   datapath_port_ips,
-                                                  metadaport_subnet_cidrs)
+                                                  metadaport_subnet_cidrs,
+                                                  lla)
         actual_cidrs_to_add, actual_cidrs_to_delete = actual_result
 
         self.assertSetEqual(actual_cidrs_to_add, expected_cidrs_to_add)
         self.assertSetEqual(actual_cidrs_to_delete, expected_cidrs_to_delete)
 
     def test__process_cidrs_when_current_namespace_contains_stale_cidr(self):
-        current_namespace_cidrs = set([n_const.METADATA_CIDR, '10.0.1.0/31'])
+        lla = 'fe80::f816:3eff:fe63:8dc5/64'
+        current_namespace_cidrs = set([n_const.METADATA_CIDR, '10.0.1.0/31',
+                                       lla])
         datapath_port_ips = ['10.0.0.2', '10.0.0.3', '10.0.1.5']
         metadaport_subnet_cidrs = ['10.0.0.0/30', '10.0.1.0/28', '11.0.1.2/24']
 
-        expected_cidrs_to_add = set(['10.0.0.0/30', '10.0.1.0/28'])
+        expected_cidrs_to_add = set(['10.0.0.0/30', '10.0.1.0/28',
+                                     n_const.METADATA_V6_CIDR])
         expected_cidrs_to_delete = set(['10.0.1.0/31'])
 
         actual_result = self.agent._process_cidrs(current_namespace_cidrs,
                                                   datapath_port_ips,
-                                                  metadaport_subnet_cidrs)
+                                                  metadaport_subnet_cidrs,
+                                                  lla)
         actual_cidrs_to_add, actual_cidrs_to_delete = actual_result
 
         self.assertSetEqual(actual_cidrs_to_add, expected_cidrs_to_add)
@@ -262,18 +278,22 @@ class TestMetadataAgent(base.BaseTestCase):
         """Current namespace cidrs contains stale cidrs and it is missing
         new required cidrs.
         """
+        lla = 'fe80::f816:3eff:fe63:8dc5/64'
         current_namespace_cidrs = set([n_const.METADATA_CIDR,
                                       '10.0.1.0/31',
-                                      '10.0.1.0/28'])
+                                      '10.0.1.0/28',
+                                      'fe77::/64',
+                                      lla])
         datapath_port_ips = ['10.0.0.2', '10.0.1.5']
         metadaport_subnet_cidrs = ['10.0.0.0/30', '10.0.1.0/28', '11.0.1.2/24']
 
-        expected_cidrs_to_add = set(['10.0.0.0/30'])
-        expected_cidrs_to_delete = set(['10.0.1.0/31'])
+        expected_cidrs_to_add = set(['10.0.0.0/30', n_const.METADATA_V6_CIDR])
+        expected_cidrs_to_delete = set(['10.0.1.0/31', 'fe77::/64'])
 
         actual_result = self.agent._process_cidrs(current_namespace_cidrs,
                                                   datapath_port_ips,
-                                                  metadaport_subnet_cidrs)
+                                                  metadaport_subnet_cidrs,
+                                                  lla)
         actual_cidrs_to_add, actual_cidrs_to_delete = actual_result
 
         self.assertSetEqual(actual_cidrs_to_add, expected_cidrs_to_add)
@@ -289,7 +309,7 @@ class TestMetadataAgent(base.BaseTestCase):
             external_ids={'name': 'neutron-{}'.format(network_id)})
 
         with mock.patch.object(
-                self.agent.sb_idl, 'get_metadata_port_network',
+                self.agent.sb_idl, 'get_metadata_port',
                 return_value=None),\
             mock.patch.object(
                 self.agent, 'teardown_datapath') as tdp:
@@ -310,7 +330,7 @@ class TestMetadataAgent(base.BaseTestCase):
                                                  '10.204.0.10/29'})
 
         with mock.patch.object(
-                self.agent.sb_idl, 'get_metadata_port_network',
+                self.agent.sb_idl, 'get_metadata_port',
                 return_value=metadadata_port),\
             mock.patch.object(
                 self.agent, 'teardown_datapath') as tdp:
@@ -330,7 +350,7 @@ class TestMetadataAgent(base.BaseTestCase):
                                    external_ids={'neutron:cidrs':
                                                  '10.204.0.10/29'})
 
-        with mock.patch.object(self.agent.sb_idl, 'get_metadata_port_network',
+        with mock.patch.object(self.agent.sb_idl, 'get_metadata_port',
                     return_value=metadadata_port),\
                 mock.patch.object(self.agent.sb_idl, 'get_ports_on_chassis',
                     return_value=datapath_ports),\
@@ -338,12 +358,12 @@ class TestMetadataAgent(base.BaseTestCase):
             self.assertIsNone(self.agent._get_provision_params(datapath))
             tdp.assert_called_once_with(network_id)
 
-    def test__get_provision_params_returns_provision_parameters(self):
+    def _test__get_provision_params_returns_provision_parameters(self,
+                                                                 port_ip):
         """The happy path when datapath has ports with "external" or ""(blank)
         types and metadata port contains MAC and subnet CIDRs.
         """
         network_id = '1'
-        port_ip = '1.2.3.4'
         metada_port_mac = "fa:16:3e:22:65:18"
         metada_port_subnet_cidr = "10.204.0.10/29"
         metada_port_logical_port = "3b66c176-199b-48ec-8331-c1fd3f6e2b44"
@@ -360,7 +380,7 @@ class TestMetadataAgent(base.BaseTestCase):
                                                  metada_port_subnet_cidr},
                                    logical_port=metada_port_logical_port)
 
-        with mock.patch.object(self.agent.sb_idl, 'get_metadata_port_network',
+        with mock.patch.object(self.agent.sb_idl, 'get_metadata_port',
                 return_value=metadadata_port),\
             mock.patch.object(self.agent.sb_idl, 'get_ports_on_chassis',
                 return_value=datapath_ports):
@@ -369,13 +389,23 @@ class TestMetadataAgent(base.BaseTestCase):
         net_name, datapath_port_ips, metadata_port_info = actual_params
 
         self.assertEqual(network_id, net_name)
-        self.assertListEqual([port_ip], datapath_port_ips)
+
+        if utils.get_ip_version(port_ip) == n_const.IP_VERSION_4:
+            self.assertListEqual([port_ip], datapath_port_ips)
         self.assertEqual(metada_port_mac, metadata_port_info.mac)
         self.assertSetEqual(set([metada_port_subnet_cidr]),
                             metadata_port_info.ip_addresses)
         self.assertEqual(metada_port_logical_port,
                          metadata_port_info.logical_port)
 
+    def test__get_provision_params_returns_provision_parameters(self):
+        self._test__get_provision_params_returns_provision_parameters(
+            '1.2.3.4')
+
+    def test__get_provision_params_returns_provision_parameters_ipv6(self):
+        self._test__get_provision_params_returns_provision_parameters(
+            'fe80::f816:3eff:feb6:c0c0')
+
     def test_provision_datapath(self):
         """Test datapath provisioning.
 
@@ -416,6 +446,7 @@ class TestMetadataAgent(base.BaseTestCase):
                     ip_wrap, 'add_veth',
                     return_value=[ip_lib.IPDevice('ip1'),
                                   ip_lib.IPDevice('ip2')]) as add_veth,\
+                mock.patch.object(linux_utils, 'delete_if_exists'), \
                 mock.patch.object(
                     driver.MetadataDriver,
                     'spawn_monitored_metadata_proxy') as spawn_mdp, \
@@ -450,13 +481,17 @@ class TestMetadataAgent(base.BaseTestCase):
                 ('external_ids', {'iface-id': metadaport_logical_port}))
             # Check that the metadata port has the IP addresses properly
             # configured and that IPv6 address has been skipped.
-            expected_call = [n_const.METADATA_CIDR, '10.0.0.1/23']
+            expected_call = [n_const.METADATA_CIDR, n_const.METADATA_V6_CIDR,
+                             '10.0.0.1/23',
+                             ip_lib.get_ipv6_lladdr('aa:bb:cc:dd:ee:ff')]
             self.assertCountEqual(expected_call,
                                   ip_addr_add_multiple.call_args.args[0])
             # Check that metadata proxy has been spawned
             spawn_mdp.assert_called_once_with(
                 mock.ANY, nemaspace_name, 80, mock.ANY,
-                bind_address=n_const.METADATA_V4_IP, network_id=net_name)
+                bind_address=n_const.METADATA_V4_IP, network_id=net_name,
+                bind_address_v6=n_const.METADATA_V6_IP,
+                bind_interface='veth_1')
             mock_checksum.assert_called_once_with(nemaspace_name)
 
     def test__load_config(self):
