@@ -23,7 +23,7 @@ from neutron.tests.unit.conf.policies import test_base as base
 class RouterAPITestCase(base.PolicyBaseTestCase):
 
     def setUp(self):
-        super(RouterAPITestCase, self).setUp()
+        super().setUp()
         self.target = {'project_id': self.project_id}
         self.alt_target = {'project_id': self.alt_project_id}
 
@@ -31,7 +31,7 @@ class RouterAPITestCase(base.PolicyBaseTestCase):
 class SystemAdminTests(RouterAPITestCase):
 
     def setUp(self):
-        super(SystemAdminTests, self).setUp()
+        super().setUp()
         self.context = self.system_admin_ctx
 
     def test_create_router(self):
@@ -138,6 +138,16 @@ class SystemAdminTests(RouterAPITestCase):
             self.context, 'create_router:enable_default_route_ecmp',
             self.alt_target)
 
+    def test_create_routers_tags(self):
+        self.assertRaises(
+            base_policy.InvalidScope,
+            policy.enforce,
+            self.context, 'create_routers_tags', self.target)
+        self.assertRaises(
+            base_policy.InvalidScope,
+            policy.enforce,
+            self.context, 'create_routers_tags', self.alt_target)
+
     def test_get_router(self):
         self.assertRaises(
             base_policy.InvalidScope,
@@ -336,21 +346,21 @@ class SystemAdminTests(RouterAPITestCase):
 class SystemMemberTests(SystemAdminTests):
 
     def setUp(self):
-        super(SystemMemberTests, self).setUp()
+        super().setUp()
         self.context = self.system_member_ctx
 
 
 class SystemReaderTests(SystemMemberTests):
 
     def setUp(self):
-        super(SystemReaderTests, self).setUp()
+        super().setUp()
         self.context = self.system_reader_ctx
 
 
 class AdminTests(RouterAPITestCase):
 
     def setUp(self):
-        super(AdminTests, self).setUp()
+        super().setUp()
         self.context = self.project_admin_ctx
 
     def test_create_router(self):
@@ -415,6 +425,13 @@ class AdminTests(RouterAPITestCase):
                 'create_router:external_gateway_info:external_fixed_ips',
                 self.alt_target))
 
+    def test_create_routers_tags(self):
+        self.assertTrue(
+            policy.enforce(self.context, 'create_routers_tags', self.target))
+        self.assertTrue(
+            policy.enforce(self.context, 'create_routers_tags',
+                           self.alt_target))
+
     def test_update_router_enable_default_route_bfd(self):
         self.assertTrue(
             policy.enforce(
@@ -564,11 +581,11 @@ class AdminTests(RouterAPITestCase):
                            'remove_router_interface', self.alt_target))
 
 
-class ProjectMemberTests(AdminTests):
+class ProjectManagerTests(AdminTests):
 
     def setUp(self):
-        super(ProjectMemberTests, self).setUp()
-        self.context = self.project_member_ctx
+        super().setUp()
+        self.context = self.project_manager_ctx
 
     def test_create_router(self):
         self.assertTrue(
@@ -646,6 +663,14 @@ class ProjectMemberTests(AdminTests):
             'create_router:external_gateway_info:external_fixed_ips',
             self.alt_target)
 
+    def test_create_routers_tags(self):
+        self.assertTrue(
+            policy.enforce(self.context, 'create_routers_tags', self.target))
+        self.assertRaises(
+            base_policy.PolicyNotAuthorized,
+            policy.enforce,
+            self.context, 'create_routers_tags', self.alt_target)
+
     def test_update_router_enable_default_route_bfd(self):
         self.assertRaises(
             base_policy.PolicyNotAuthorized,
@@ -829,10 +854,17 @@ class ProjectMemberTests(AdminTests):
             self.context, 'remove_router_interface', self.alt_target)
 
 
+class ProjectMemberTests(ProjectManagerTests):
+
+    def setUp(self):
+        super().setUp()
+        self.context = self.project_member_ctx
+
+
 class ProjectReaderTests(ProjectMemberTests):
 
     def setUp(self):
-        super(ProjectMemberTests, self).setUp()
+        super().setUp()
         self.context = self.project_reader_ctx
 
     def test_create_router(self):
@@ -869,6 +901,16 @@ class ProjectReaderTests(ProjectMemberTests):
             self.context, 'create_router:external_gateway_info:network_id',
             self.alt_target)
 
+    def test_create_routers_tags(self):
+        self.assertRaises(
+            base_policy.PolicyNotAuthorized,
+            policy.enforce,
+            self.context, 'create_routers_tags', self.target)
+        self.assertRaises(
+            base_policy.PolicyNotAuthorized,
+            policy.enforce,
+            self.context, 'create_routers_tags', self.alt_target)
+
     def test_update_router(self):
         self.assertRaises(
             base_policy.PolicyNotAuthorized,
@@ -957,7 +999,7 @@ class ProjectReaderTests(ProjectMemberTests):
 class ExtrarouteAPITestCase(base.PolicyBaseTestCase):
 
     def setUp(self):
-        super(ExtrarouteAPITestCase, self).setUp()
+        super().setUp()
         self.router = {
             'id': uuidutils.generate_uuid(),
             'project_id': self.project_id}
@@ -975,7 +1017,7 @@ class ExtrarouteAPITestCase(base.PolicyBaseTestCase):
 class SystemAdminExtrarouteTests(ExtrarouteAPITestCase):
 
     def setUp(self):
-        super(SystemAdminExtrarouteTests, self).setUp()
+        super().setUp()
         self.context = self.system_admin_ctx
 
     def test_add_extraroute(self):
@@ -1002,21 +1044,21 @@ class SystemAdminExtrarouteTests(ExtrarouteAPITestCase):
 class SystemMemberExtrarouteTests(SystemAdminExtrarouteTests):
 
     def setUp(self):
-        super(SystemMemberExtrarouteTests, self).setUp()
+        super().setUp()
         self.context = self.system_member_ctx
 
 
 class SystemReaderExtrarouteTests(SystemMemberExtrarouteTests):
 
     def setUp(self):
-        super(SystemReaderExtrarouteTests, self).setUp()
+        super().setUp()
         self.context = self.system_reader_ctx
 
 
 class AdminExtrarouteTests(ExtrarouteAPITestCase):
 
     def setUp(self):
-        super(AdminExtrarouteTests, self).setUp()
+        super().setUp()
         self.context = self.project_admin_ctx
 
     def test_add_extraroute(self):
@@ -1036,7 +1078,7 @@ class AdminExtrarouteTests(ExtrarouteAPITestCase):
 class ProjectMemberExtrarouteTests(AdminExtrarouteTests):
 
     def setUp(self):
-        super(ProjectMemberExtrarouteTests, self).setUp()
+        super().setUp()
         self.context = self.project_member_ctx
 
     def test_add_extraroute(self):
@@ -1059,7 +1101,7 @@ class ProjectMemberExtrarouteTests(AdminExtrarouteTests):
 class ProjectReaderExtrarouteTests(ProjectMemberExtrarouteTests):
 
     def setUp(self):
-        super(ProjectReaderExtrarouteTests, self).setUp()
+        super().setUp()
         self.context = self.project_reader_ctx
 
     def test_add_extraroute(self):
@@ -1081,3 +1123,146 @@ class ProjectReaderExtrarouteTests(ProjectMemberExtrarouteTests):
             base_policy.PolicyNotAuthorized,
             policy.enforce,
             self.context, 'remove_extraroutes', self.alt_target)
+
+
+class ServiceRoleTests(RouterAPITestCase):
+
+    def setUp(self):
+        super().setUp()
+        self.context = self.service_ctx
+
+    def test_create_router(self):
+        self.assertRaises(
+            base_policy.PolicyNotAuthorized,
+            policy.enforce,
+            self.context, 'create_router', self.target)
+
+    def test_create_router_distributed(self):
+        self.assertRaises(
+            base_policy.PolicyNotAuthorized,
+            policy.enforce,
+            self.context, 'create_router:distributed', self.target)
+
+    def test_create_router_ha(self):
+        self.assertRaises(
+            base_policy.PolicyNotAuthorized,
+            policy.enforce,
+            self.context, 'create_router:ha', self.target)
+
+    def test_create_router_external_gateway_info(self):
+        self.assertRaises(
+            base_policy.PolicyNotAuthorized,
+            policy.enforce,
+            self.context, 'create_router:external_gateway_info',
+            self.target)
+
+    def test_create_router_external_gateway_info_network_id(self):
+        self.assertRaises(
+            base_policy.PolicyNotAuthorized,
+            policy.enforce,
+            self.context, 'create_router:external_gateway_info:network_id',
+            self.target)
+
+    def test_create_router_external_gateway_info_enable_snat(self):
+        self.assertRaises(
+            base_policy.PolicyNotAuthorized,
+            policy.enforce,
+            self.context, 'create_router:external_gateway_info:enable_snat',
+            self.target)
+
+    def test_create_router_external_gateway_info_external_fixed_ips(self):
+        self.assertRaises(
+            base_policy.PolicyNotAuthorized,
+            policy.enforce,
+            self.context,
+            'create_router:external_gateway_info:external_fixed_ips',
+            self.target)
+
+    def test_create_routers_tags(self):
+        self.assertRaises(
+            base_policy.PolicyNotAuthorized,
+            policy.enforce,
+            self.context, 'create_routers_tags', self.target)
+
+    def test_get_router(self):
+        self.assertRaises(
+            base_policy.PolicyNotAuthorized,
+            policy.enforce,
+            self.context, 'get_router', self.target)
+
+    def test_get_router_distributed(self):
+        self.assertRaises(
+            base_policy.PolicyNotAuthorized,
+            policy.enforce,
+            self.context, 'get_router:distributed', self.target)
+
+    def test_get_router_ha(self):
+        self.assertRaises(
+            base_policy.PolicyNotAuthorized,
+            policy.enforce,
+            self.context, 'get_router:ha', self.target)
+
+    def test_update_router(self):
+        self.assertRaises(
+            base_policy.PolicyNotAuthorized,
+            policy.enforce,
+            self.context, 'update_router', self.target)
+
+    def test_update_router_distributed(self):
+        self.assertRaises(
+            base_policy.PolicyNotAuthorized,
+            policy.enforce,
+            self.context, 'update_router:distributed', self.target)
+
+    def test_update_router_ha(self):
+        self.assertRaises(
+            base_policy.PolicyNotAuthorized,
+            policy.enforce,
+            self.context, 'update_router:ha', self.target)
+
+    def test_update_router_external_gateway_info(self):
+        self.assertRaises(
+            base_policy.PolicyNotAuthorized,
+            policy.enforce,
+            self.context, 'update_router:external_gateway_info',
+            self.target)
+
+    def test_update_router_external_gateway_info_network_id(self):
+        self.assertRaises(
+            base_policy.PolicyNotAuthorized,
+            policy.enforce,
+            self.context, 'update_router:external_gateway_info:network_id',
+            self.target)
+
+    def test_update_router_external_gateway_info_enable_snat(self):
+        self.assertRaises(
+            base_policy.PolicyNotAuthorized,
+            policy.enforce,
+            self.context, 'update_router:external_gateway_info:enable_snat',
+            self.target)
+
+    def test_update_router_external_gateway_info_external_fixed_ips(self):
+        self.assertRaises(
+            base_policy.PolicyNotAuthorized,
+            policy.enforce,
+            self.context,
+            'update_router:external_gateway_info:external_fixed_ips',
+            self.target)
+
+    def test_delete_router(self):
+        self.assertRaises(
+            base_policy.PolicyNotAuthorized,
+            policy.enforce,
+            self.context, 'delete_router', self.target)
+
+    def test_add_router_interface(self):
+        self.assertRaises(
+            base_policy.PolicyNotAuthorized,
+            policy.enforce,
+            self.context, 'add_router_interface', self.target)
+
+    def test_remove_router_interface(self):
+        self.assertRaises(
+            base_policy.PolicyNotAuthorized,
+            policy.enforce,
+            self.context, 'remove_router_interface', self.target)
