@@ -13,7 +13,6 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import copy
 from unittest import mock
 
 from oslo_policy import policy as base_policy
@@ -26,7 +25,7 @@ from neutron.tests.unit.conf.policies import test_base as base
 class SecurityGroupAPITestCase(base.PolicyBaseTestCase):
 
     def setUp(self):
-        super(SecurityGroupAPITestCase, self).setUp()
+        super().setUp()
         self.target = {'project_id': self.project_id}
         self.alt_target = {'project_id': self.alt_project_id}
 
@@ -34,7 +33,7 @@ class SecurityGroupAPITestCase(base.PolicyBaseTestCase):
 class SystemAdminSecurityGroupTests(SecurityGroupAPITestCase):
 
     def setUp(self):
-        super(SystemAdminSecurityGroupTests, self).setUp()
+        super().setUp()
         self.context = self.system_admin_ctx
 
     def test_create_security_group(self):
@@ -47,6 +46,16 @@ class SystemAdminSecurityGroupTests(SecurityGroupAPITestCase):
             policy.enforce,
             self.context, 'create_security_group', self.alt_target)
 
+    def test_create_security_groups_tags(self):
+        self.assertRaises(
+            base_policy.InvalidScope,
+            policy.enforce,
+            self.context, 'create_security_groups_tags', self.target)
+        self.assertRaises(
+            base_policy.InvalidScope,
+            policy.enforce,
+            self.context, 'create_security_groups_tags', self.alt_target)
+
     def test_get_security_group(self):
         self.assertRaises(
             base_policy.InvalidScope,
@@ -111,21 +120,21 @@ class SystemAdminSecurityGroupTests(SecurityGroupAPITestCase):
 class SystemMemberSecurityGroupTests(SystemAdminSecurityGroupTests):
 
     def setUp(self):
-        super(SystemMemberSecurityGroupTests, self).setUp()
+        super().setUp()
         self.context = self.system_member_ctx
 
 
 class SystemReaderSecurityGroupTests(SystemMemberSecurityGroupTests):
 
     def setUp(self):
-        super(SystemReaderSecurityGroupTests, self).setUp()
+        super().setUp()
         self.context = self.system_reader_ctx
 
 
 class AdminSecurityGroupTests(SecurityGroupAPITestCase):
 
     def setUp(self):
-        super(AdminSecurityGroupTests, self).setUp()
+        super().setUp()
         self.context = self.project_admin_ctx
 
     def test_create_security_group(self):
@@ -135,6 +144,14 @@ class AdminSecurityGroupTests(SecurityGroupAPITestCase):
             policy.enforce(
                 self.context, 'create_security_group', self.alt_target))
 
+    def test_create_security_groups_tags(self):
+        self.assertTrue(
+            policy.enforce(self.context, 'create_security_groups_tags',
+                           self.target))
+        self.assertTrue(
+            policy.enforce(self.context, 'create_security_groups_tags',
+                           self.alt_target))
+
     def test_get_security_group(self):
         self.assertTrue(
             policy.enforce(self.context, 'get_security_group', self.target))
@@ -181,11 +198,11 @@ class AdminSecurityGroupTests(SecurityGroupAPITestCase):
                            self.alt_target))
 
 
-class ProjectMemberSecurityGroupTests(AdminSecurityGroupTests):
+class ProjectManagerSecurityGroupTests(AdminSecurityGroupTests):
 
     def setUp(self):
-        super(ProjectMemberSecurityGroupTests, self).setUp()
-        self.context = self.project_member_ctx
+        super().setUp()
+        self.context = self.project_manager_ctx
 
     def test_create_security_group(self):
         self.assertTrue(
@@ -195,6 +212,15 @@ class ProjectMemberSecurityGroupTests(AdminSecurityGroupTests):
             policy.enforce,
             self.context, 'create_security_group', self.alt_target)
 
+    def test_create_security_groups_tags(self):
+        self.assertTrue(
+            policy.enforce(self.context, 'create_security_groups_tags',
+                           self.target))
+        self.assertRaises(
+            base_policy.PolicyNotAuthorized,
+            policy.enforce,
+            self.context, 'create_security_groups_tags', self.alt_target)
+
     def test_get_security_group(self):
         self.assertTrue(
             policy.enforce(self.context, 'get_security_group', self.target))
@@ -245,10 +271,17 @@ class ProjectMemberSecurityGroupTests(AdminSecurityGroupTests):
             self.context, 'delete_security_groups_tags', self.alt_target)
 
 
+class ProjectMemberSecurityGroupTests(ProjectManagerSecurityGroupTests):
+
+    def setUp(self):
+        super().setUp()
+        self.context = self.project_member_ctx
+
+
 class ProjectReaderSecurityGroupTests(ProjectMemberSecurityGroupTests):
 
     def setUp(self):
-        super(ProjectReaderSecurityGroupTests, self).setUp()
+        super().setUp()
         self.context = self.project_reader_ctx
 
     def test_create_security_group(self):
@@ -261,6 +294,16 @@ class ProjectReaderSecurityGroupTests(ProjectMemberSecurityGroupTests):
             policy.enforce,
             self.context, 'create_security_group', self.alt_target)
 
+    def test_create_security_groups_tags(self):
+        self.assertRaises(
+            base_policy.PolicyNotAuthorized,
+            policy.enforce,
+            self.context, 'create_security_groups_tags', self.target)
+        self.assertRaises(
+            base_policy.PolicyNotAuthorized,
+            policy.enforce,
+            self.context, 'create_security_groups_tags', self.alt_target)
+
     def test_update_security_group(self):
         self.assertRaises(
             base_policy.PolicyNotAuthorized,
@@ -305,7 +348,7 @@ class ProjectReaderSecurityGroupTests(ProjectMemberSecurityGroupTests):
 class ServiceRoleSecurityGroupTests(SecurityGroupAPITestCase):
 
     def setUp(self):
-        super(ServiceRoleSecurityGroupTests, self).setUp()
+        super().setUp()
         self.context = self.service_ctx
 
     def test_create_security_group(self):
@@ -314,6 +357,12 @@ class ServiceRoleSecurityGroupTests(SecurityGroupAPITestCase):
             policy.enforce,
             self.context, 'create_security_group', self.target)
 
+    def test_create_security_groups_tags(self):
+        self.assertRaises(
+            base_policy.PolicyNotAuthorized,
+            policy.enforce,
+            self.context, 'create_security_groups_tags', self.target)
+
     def test_get_security_group(self):
         self.assertRaises(
             base_policy.PolicyNotAuthorized,
@@ -336,38 +385,22 @@ class ServiceRoleSecurityGroupTests(SecurityGroupAPITestCase):
 class SecurityGroupRuleAPITestCase(base.PolicyBaseTestCase):
 
     def setUp(self):
-        super(SecurityGroupRuleAPITestCase, self).setUp()
+        super().setUp()
         self.sg = {
             'id': uuidutils.generate_uuid(),
-            'project_id': self.project_id,
-            'tenant_id': self.project_id}
-        self.alt_sg = {
-            'id': uuidutils.generate_uuid(),
-            'project_id': self.alt_project_id,
-            'tenant_id': self.alt_project_id}
+            'project_id': self.project_id}
 
         self.target = {
             'project_id': self.project_id,
-            'tenant_id': self.project_id,
             'security_group_id': self.sg['id'],
-            'ext_parent:tenant_id': self.sg['id'],
             'ext_parent_security_group_id': self.sg['id']}
         self.alt_target = {
             'project_id': self.alt_project_id,
-            'tenant_id': self.alt_project_id,
-            'security_group_id': self.alt_sg['id'],
-            'ext_parent:tenant_id': self.alt_sg['id'],
-            'ext_parent_security_group_id': self.alt_sg['id']}
-
-        def get_security_group_mock(context, id,
-                                    fields=None, tenant_id=None):
-            if id == self.alt_sg['id']:
-                return self.alt_sg
-            return self.sg
+            'security_group_id': self.sg['id'],
+            'ext_parent_security_group_id': self.sg['id']}
 
         self.plugin_mock = mock.Mock()
-        self.plugin_mock.get_security_group.side_effect = (
-            get_security_group_mock)
+        self.plugin_mock.get_security_group.return_value = self.sg
         mock.patch(
             'neutron_lib.plugins.directory.get_plugin',
             return_value=self.plugin_mock).start()
@@ -394,7 +427,7 @@ class SecurityGroupRuleAPITestCase(base.PolicyBaseTestCase):
 class SystemAdminSecurityGroupRuleTests(SecurityGroupRuleAPITestCase):
 
     def setUp(self):
-        super(SystemAdminSecurityGroupRuleTests, self).setUp()
+        super().setUp()
         self.context = self.system_admin_ctx
 
     def test_create_security_group_rule(self):
@@ -431,21 +464,21 @@ class SystemAdminSecurityGroupRuleTests(SecurityGroupRuleAPITestCase):
 class SystemMemberSecurityGroupRuleTests(SystemAdminSecurityGroupRuleTests):
 
     def setUp(self):
-        super(SystemMemberSecurityGroupRuleTests, self).setUp()
+        super().setUp()
         self.context = self.system_member_ctx
 
 
 class SystemReaderSecurityGroupRuleTests(SystemMemberSecurityGroupRuleTests):
 
     def setUp(self):
-        super(SystemReaderSecurityGroupRuleTests, self).setUp()
+        super().setUp()
         self.context = self.system_reader_ctx
 
 
 class AdminSecurityGroupRuleTests(SecurityGroupRuleAPITestCase):
 
     def setUp(self):
-        super(AdminSecurityGroupRuleTests, self).setUp()
+        super().setUp()
         self.context = self.project_admin_ctx
 
     def test_create_security_group_rule(self):
@@ -491,11 +524,11 @@ class AdminSecurityGroupRuleTests(SecurityGroupRuleAPITestCase):
                            'delete_security_group_rule', self.alt_target))
 
 
-class ProjectMemberSecurityGroupRuleTests(AdminSecurityGroupRuleTests):
+class ProjectManagerSecurityGroupRuleTests(AdminSecurityGroupRuleTests):
 
     def setUp(self):
-        super(ProjectMemberSecurityGroupRuleTests, self).setUp()
-        self.context = self.project_member_ctx
+        super().setUp()
+        self.context = self.project_manager_ctx
 
     def test_create_security_group_rule(self):
         self.assertTrue(
@@ -506,17 +539,6 @@ class ProjectMemberSecurityGroupRuleTests(AdminSecurityGroupRuleTests):
             policy.enforce,
             self.context, 'create_security_group_rule', self.alt_target)
 
-        # Test for the SG_OWNER different then current user case:
-        target = copy.copy(self.target)
-        target['security_group_id'] = self.alt_sg['id']
-        target['ext_parent:tenant_id'] = self.alt_sg['tenant_id']
-        target['ext_parent_security_group_id'] = self.alt_sg['id']
-        self.plugin_mock.get_security_group.return_value = self.alt_sg
-        self.assertRaises(
-            base_policy.PolicyNotAuthorized,
-            policy.enforce,
-            self.context, 'create_security_group_rule', target)
-
     def test_create_security_group_rule_default_sg(self):
         self.override_create_security_group_rule()
         self.assertRaises(
@@ -541,23 +563,11 @@ class ProjectMemberSecurityGroupRuleTests(AdminSecurityGroupRuleTests):
         self.assertTrue(
             policy.enforce(self.context,
                            'delete_security_group_rule', self.target))
-        self.plugin_mock.get_security_group.return_value = self.alt_sg
         self.assertRaises(
             base_policy.PolicyNotAuthorized,
             policy.enforce,
             self.context, 'delete_security_group_rule', self.alt_target)
 
-        # Test for the SG_OWNER different then current user case:
-        target = copy.copy(self.target)
-        target['security_group_id'] = self.alt_sg['id']
-        target['ext_parent:tenant_id'] = self.alt_sg['tenant_id']
-        target['ext_parent_security_group_id'] = self.alt_sg['id']
-        self.plugin_mock.get_security_group.return_value = self.alt_sg
-        self.assertRaises(
-            base_policy.PolicyNotAuthorized,
-            policy.enforce,
-            self.context, 'delete_security_group_rule', target)
-
     def test_delete_security_group_rule_default_sg(self):
         self.override_delete_security_group_rule()
         self.assertRaises(
@@ -570,10 +580,18 @@ class ProjectMemberSecurityGroupRuleTests(AdminSecurityGroupRuleTests):
             self.context, 'delete_security_group_rule', self.alt_target)
 
 
+class ProjectMemberSecurityGroupRuleTests(
+        ProjectManagerSecurityGroupRuleTests):
+
+    def setUp(self):
+        super().setUp()
+        self.context = self.project_member_ctx
+
+
 class ProjectReaderSecurityGroupRuleTests(ProjectMemberSecurityGroupRuleTests):
 
     def setUp(self):
-        super(ProjectReaderSecurityGroupRuleTests, self).setUp()
+        super().setUp()
         self.context = self.project_reader_ctx
 
     def test_create_security_group_rule(self):
@@ -585,16 +603,6 @@ class ProjectReaderSecurityGroupRuleTests(ProjectMemberSecurityGroupRuleTests):
             base_policy.PolicyNotAuthorized,
             policy.enforce,
             self.context, 'create_security_group_rule', self.alt_target)
-        # Test for the SG_OWNER different then current user case:
-        target = copy.copy(self.target)
-        target['security_group_id'] = self.alt_sg['id']
-        target['ext_parent:tenant_id'] = self.alt_sg['tenant_id']
-        target['ext_parent_security_group_id'] = self.alt_sg['id']
-        self.plugin_mock.get_security_group.return_value = self.alt_sg
-        self.assertRaises(
-            base_policy.PolicyNotAuthorized,
-            policy.enforce,
-            self.context, 'create_security_group_rule', target)
 
     def test_delete_security_group_rule(self):
         self.assertRaises(
@@ -605,22 +613,12 @@ class ProjectReaderSecurityGroupRuleTests(ProjectMemberSecurityGroupRuleTests):
             base_policy.PolicyNotAuthorized,
             policy.enforce,
             self.context, 'delete_security_group_rule', self.alt_target)
-        # Test for the SG_OWNER different then current user case:
-        target = copy.copy(self.target)
-        target['security_group_id'] = self.alt_sg['id']
-        target['ext_parent:tenant_id'] = self.alt_sg['tenant_id']
-        target['ext_parent_security_group_id'] = self.alt_sg['id']
-        self.plugin_mock.get_security_group.return_value = self.alt_sg
-        self.assertRaises(
-            base_policy.PolicyNotAuthorized,
-            policy.enforce,
-            self.context, 'delete_security_group_rule', target)
 
 
 class ServiceRoleSecurityGroupRuleTests(SecurityGroupRuleAPITestCase):
 
     def setUp(self):
-        super(ServiceRoleSecurityGroupRuleTests, self).setUp()
+        super().setUp()
         self.context = self.service_ctx
 
     def test_create_security_group_rule(self):
