@@ -73,7 +73,8 @@ object_data = {
     'NetworkSegment': '1.0-57b7f2960971e3b95ded20cbc59244a8',
     'NetworkSegmentRange': '1.0-bdec1fffc9058ea676089b1f2f2b3cf3',
     'NetworkSubnetLock': '1.0-140de39d4b86ae346dc3d70b885bea53',
-    'Port': '1.8-1aa850ab5529128de07e82c6fb75fcb5',
+    'Port': '1.10-ae84f686bfc3deb4017495134da6ef04',
+    'PortHardwareOffloadType': '1.0-5f424d02b144fd1832ac3e6b03662674',
     'PortDeviceProfile': '1.0-b98c7083cc3e93d176fd7a91ae13af32',
     'PortHints': '1.0-9ebf6e12fa427809476a92c7432352b8',
     'PortNumaAffinityPolicy': '1.0-38fcea43e7bfb2536461f3d053c43aa3',
@@ -84,6 +85,7 @@ object_data = {
     'PortForwarding': '1.3-402b1fb5a754808b82a966c95f468113',
     'PortSecurity': '1.0-b30802391a87945ee9c07582b4ff95e3',
     'PortUplinkStatusPropagation': '1.1-f0a4ca451a941910376c33616dea5de2',
+    'PortTrusted': '1.0-8312fb91937412cdeb92c3279059c7ce',
     'ProviderResourceAssociation': '1.0-05ab2d5a3017e5ce9dd381328f285f34',
     'ProvisioningBlock': '1.0-c19d6d05bfa8143533471c1296066125',
     'QosBandwidthLimitRule': '1.5-51b662b12a8d1dfa89288d826c6d26d3',
@@ -111,15 +113,15 @@ object_data = {
     'RouterNDPProxyState': '1.0-4042e475bf173d1d8d17adb962eae1b2',
     'RouterPort': '1.0-c8c8f499bcdd59186fcd83f323106908',
     'RouterRoute': '1.0-07fc5337c801fb8c6ccfbcc5afb45907',
-    'SecurityGroup': '1.5-7eb8e44c327512e7bb1759ab41ede44b',
+    'SecurityGroup': '1.6-7eb8e44c327512e7bb1759ab41ede44b',
     'SecurityGroupDefaultRule': '1.0-d498fd4993b6732f3f266c4b7e292e22',
     'SecurityGroupPortBinding': '1.0-6879d5c0af80396ef5a72934b6a6ef20',
     'SecurityGroupRBAC': '1.1-be82ed54376b85ee4f963d479ac48c91',
-    'SecurityGroupRule': '1.2-27793368d4ac35f2ed6e0bb653c6aaad',
+    'SecurityGroupRule': '1.3-60cdd6434e35979d2b280ed28a9598d3',
     'SegmentHostMapping': '1.0-521597cf82ead26217c3bd10738f00f0',
     'ServiceProfile': '1.0-9beafc9e7d081b8258f3c5cb66ac5eed',
     'StandardAttribute': '1.0-617d4f46524c4ce734a6fc1cc0ac6a0b',
-    'Subnet': '1.1-5b7e1789a1732259d1e28b4bd87eb1c2',
+    'Subnet': '1.2-476759b72624961bb4aade8203289227',
     'SubnetDNSPublishFixedIP': '1.0-db22af6fa20b143986f0cbe06cbfe0ea',
     'SubnetPool': '1.1-a0e03895d1a6e7b9d4ab7b0ca13c3867',
     'SubnetPoolPrefix': '1.0-13c15144135eb869faa4a76dc3ee3b6c',
