@@ -137,7 +137,8 @@ class TestOVNClient(TestOVNClientBase):
             maintain_bfd=False,
             external_ids={
                 'neutron:is_ext_gw': 'true',
-                'neutron:subnet_id': subnet['id']})
+                'neutron:subnet_id': subnet['id'],
+                constants.OVN_LRSR_EXT_ID_KEY: 'true'})
 
     def test__add_router_ext_gw_default_route_ecmp(self):
         plugin = mock.MagicMock()
@@ -187,7 +188,8 @@ class TestOVNClient(TestOVNClientBase):
                       maintain_bfd=False,
                       external_ids={
                          'neutron:is_ext_gw': 'true',
-                         'neutron:subnet_id': subnet1['id']},
+                         'neutron:subnet_id': subnet1['id'],
+                         constants.OVN_LRSR_EXT_ID_KEY: 'true'},
                       ),
             mock.call('neutron-' + router['id'],
                       ip_prefix='0.0.0.0/0',
@@ -195,7 +197,8 @@ class TestOVNClient(TestOVNClientBase):
                       maintain_bfd=False,
                       external_ids={
                          'neutron:is_ext_gw': 'true',
-                         'neutron:subnet_id': subnet2['id']},
+                         'neutron:subnet_id': subnet2['id'],
+                         constants.OVN_LRSR_EXT_ID_KEY: 'true'},
                       ),
         ])
 
@@ -233,12 +236,6 @@ class TestOVNClient(TestOVNClientBase):
             self.ovn_client._add_router_ext_gw(mock.Mock(), router, txn))
         self.nb_idl.add_static_route.assert_not_called()
 
-    def test_checkout_ip_list(self):
-        addresses = ["192.168.2.2/32", "2001:db8::/32"]
-        add_map = self.ovn_client._checkout_ip_list(addresses)
-        self.assertEqual(["192.168.2.2/32"], add_map[const.IP_VERSION_4])
-        self.assertEqual(["2001:db8::/32"], add_map[const.IP_VERSION_6])
-
     def test_update_lsp_host_info_up(self):
         context = mock.MagicMock()
         host_id = 'fake-binding-host-id'
